#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include <math.h>
#include<stdio.h>
#include<stdlib.h>

#define RQ_SET_LED    0
#define RQ_SET_7SEG   1
#define RQ_GET_SWITCH 2

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

int seg7=0;

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == RQ_SET_7SEG)
    {
        seg7 = (uint8_t) rq->wValue.bytes[0] + ((uint8_t) rq->wValue.bytes[1])* 256 ;
        return 0;
    }
    else if (rq->bRequest == RQ_GET_SWITCH)
    {
        if ((PINC & (1<<PC3)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }

    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */
void show(int8_t a,int8_t b) {
	int8_t tmp[4]={0b01100010,0b01100001,0b01000011,0b00100011};
	int8_t x[4]={0b000001,0b000010,0b000100,0b001000};
	PORTD&=tmp[b-1];
	PORTB|=x[a-1]+16;
}

int8_t step1a=0,step1b=0;
void step1(int8_t a,int8_t b) {
	show(step1a+1,step1b+1);
	step1a+=a;
	step1b+=b;
	if(step1a==4||step1a==-1) step1b+=a;
	else if(step1b==4||step1b==-1) step1a+=b;
	step1a=(step1a+4)%4;
	step1b=(step1b+4)%4;
}

int8_t step2a=0,step2b=0;
void step2(int8_t a) {
	show(1,step2a+1);
	show(2,step2a+1);
	show(3,step2a+1);
	show(4,step2a+1);
	step2a+=a;
	step2a%=4;
}

int8_t step3a=0,i3=0,j3=0;
/*int8_t step3b[16][2]={{1,1},{1,2},{1,3},{1,4}/,{2,4},{3,4},{4,4},/
					{4,3},{4,2},{4,1}/,{3,1},{2,1},/{2,2},{2,3},/
					{3,3}/,{2,3}};*/
void step3() {
	if(step3a>15) {i3=0;j3=0;}
	else if(step3a>14) j3--;
	else if(step3a>13) i3++;
	else if(step3a>11) j3++;
	else if(step3a>9) i3--;
	else if(step3a>6) j3--;
	else if(step3a>3) i3++;
	else if(step3a>0) j3++;  
	show(i3+1,j3+1);
	step3a++;
	step3a%=17;
}

int8_t step4a=0;
void step4() {
	if(step4a) { show(1,1);show(2,2);show(3,3);show(4,4);}
	step4a++;
	step4a%=2;
}
void clear() {
	PORTB=16;
	PORTD=0b01100011;
}
int pow10(int n)
{
    int i=0;
    int sum=1;
    for(;i<n;i++)
     sum*=10;
    return sum;
}
void _7seg()
{
  int i;
  for(i=0;i<4;i++)
        {
          PORTC = 16*i + (seg7/pow10(i))%10  ;
          _delay_ms(1.0);
        }
	PORTC=127;
}
int main(void)
{
    DDRC  = 127; // PC0..PC2 = Output, PC3..PC5 = Input
    DDRD = 0b01100011;  //0b01100011
	PORTD = 0;
	
	DDRB =  0b001111;
	PORTB = 0b010000;
    
    usbInit();
    
    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();
    int8_t x,j=0;
    seg7=50;
    /* main event loop */
    for(;;)
    {
  
        usbPoll();
        switch(j) {
        	case 0:_7seg();break;
        	case 1:step1(0,1);break;
        	case 2:step2(1);break;
        	case 3:step3();break;
        	case 4:step4();break;
        }
        if ((PINB & (0b010000)) == 0) 
        {
        	while((PINB & (0b010000)) == 0);
        	j++;
        	clear(); 
        }
        j%=5;
        if(j!=0) for(x=0;x<seg7;x++) _delay_ms(1);
        clear();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
